----------------------------------------------------------------------------------
-- Company: 		jrok
-- Engineer: 	   jrok
-- 
-- Create Date:    
-- Design Name: 
-- Module Name:    	def_io - Behavioral 
-- Project Name: 	Defender I/O boardz
-- Target Devices: 
-- Tool versions: 
-- Description:
-- 	
--		WMS Defender I/O board
-- 	Testing the use of Xilinx CPLD to replace Defender 6821 & logic on defender
--  	I/O board
--
-- Dependencies: 
--
-- Revision: 
--	 0.00 - Aug 2007
--		initial version
--  0.01 - Dec 4 2007 
--		added the button names to the inputs
-- 
-- Additional Comments: 
--
--
--  The CB2 control output for the 6821 is simluated to swap inputs for the cocktail 
--  player 1 or player 2 
--
--  W1 upright/cocktail disables the simulated CB2 in upright mode ( input low  )
--  so always the 'B' control inputs are passed on a read
--
-- To Do
--  
-- 
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity def_io is
    Port ( 	
		db			: inout  STD_LOGIC_VECTOR (7 downto 0);
		rs 		: in STD_LOGIC_VECTOR (1 downto 0);
		a2 		: in STD_LOGIC;
		a3 		: in STD_LOGIC;
		ipia 		: in STD_LOGIC;
		rw 		: in STD_LOGIC;
		e  		: in STD_LOGIC;
		reset 	: in STD_LOGIC;
		
		pa0_a 	: in STD_LOGIC; -- fire 2p
		pa1_a 	: in STD_LOGIC; -- thrust 2p
		pa2_a 	: in STD_LOGIC; -- smart bomb 2p
		pa3_a 	: in STD_LOGIC;

		pa4		: in STD_LOGIC; -- 2p start
		pa5		: in STD_LOGIC; -- 1p start
                
		pa6_a 	: in STD_LOGIC; -- reverse 2p
		pa7_a 	: in STD_LOGIC; -- down 2p

		pa0_b 	: in STD_LOGIC; -- fire 1p
		pa1_b 	: in STD_LOGIC; -- thrust 1p
		pa2_b 	: in STD_LOGIC; -- smart bomb 1p
		pa3_b 	: in STD_LOGIC; -- hyperspace 1p

		pa6_b 	: in STD_LOGIC; -- reverse 1p
		pa7_b 	: in STD_LOGIC; -- down 1p
                
		pb0_a 	: in STD_LOGIC; -- up 2p
		pb0_b 	: in STD_LOGIC; -- up 1p

			
		W1_upright_cocktail 	: in STD_LOGIC
			  
			  
			  );
end def_io;

architecture Behavioral of def_io is

signal a_or_b_sel 	: std_logic;
signal wr_clk		 	: std_logic;

begin

-- -------------------------------------------------------------------
-- write handling
-- -------------------------------------------------------------------
process( db(5),db(4),db(3), reset, wr_clk )
begin

-- rs1/rs0 = 01 => CR_A
-- rs1/rs0 = 10 => CR_B

if ( reset ='0') then

	a_or_b_sel <= '0';

else

--	write to control reg B
-- only the CB2 bit is important

			if falling_edge( wr_clk	) then
			
				if ( db(4)='1' and db(5)='1' ) then
					a_or_b_sel <= db(3);
				end if;

			end if;

end if;


end process;

--				end if;


-- -------------------------------------------------------------------
-- read handling
-- -------------------------------------------------------------------
process( e, rs, a2,a3, ipia, rw, a_or_b_sel, reset, W1_upright_cocktail,
			pa0_a, pa1_a, pa2_a, pa3_a, pa6_a, pa7_a, pa0_b, pa1_b, pa2_b, 
			pa3_b, pa6_b, pa7_b, pa4, pa5, pb0_a, pb0_b )
begin


-- read inputs

 if 	( ipia 	= '1' ) and 
		( rw 		= '1' ) and 
		( a2		= '1' ) and 
		( a3		= '0' ) and
		( reset	= '1' ) and
		( e 		= '1' )
	then
	
	-- which internal chip address to read
	case (rs) is

	-- if upright then always pass the old multiplexed 'b' inputs
	-- 'a' inputs only used on Cocktail machine

		when "00" =>
			if ( a_or_b_sel = '0' and W1_upright_cocktail = '1' ) then
				db(0) <= not( pa0_a );
				db(1) <= not( pa1_a );
				db(2) <= not( pa2_a );
				db(3) <= not( pa3_a );
				
				db(6) <= not( pa6_a );
				db(7) <= not( pa7_a );
			else
				db(0) <= not( pa0_b );
				db(1) <= not( pa1_b );
				db(2) <= not( pa2_b );
				db(3) <= not( pa3_b );
				
				db(6) <= not( pa6_b );
				db(7) <= not( pa7_b );
				
			end if;
	
			db(4) <= not(pa4);
			db(5) <= not(pa5);

		when "01" =>
			db( 7 downto 0 ) <= "00000000";

		when "10" =>
				-- read port B
			if ( a_or_b_sel = '0' and W1_upright_cocktail = '1' ) then
				db(0) <= not( pb0_a);
			else
				db(0) <= not( pb0_b);
			end if;
	
			db(6 downto 1) <= "000000";
			db(7) <= W1_upright_cocktail;
	
		when "11" =>
			db(3) <= a_or_b_sel;
			db(2 downto 0 ) <= "000";
			db(7 downto 4 ) <= "0000";

		when others =>
			null;

	end case;

else
		db<="ZZZZZZZZ";
end if;	
	

end process;

-- clocked input for write to the control reg B
-- ignore any other writes

wr_clk	 <= ipia and rs(1) and rs(0) and a2 and not(a3) and not(rw) and e;
	

end Behavioral;
